-------------------------------------------------------------------------------
-- Blizzard Finnegan
-- 8-bit 4 function calc w/ memory [top-level design]
-------------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity top is
  port(
  clk             : in  std_logic;
  reset_n         : in  std_logic;
  exec_btn        : in  std_logic;
  input_switches  : in  std_logic_vector(7 downto 0);
  hundreds_disp   : out std_logic_vector(6 downto 0);
  tens_disp       : out std_logic_vector(6 downto 0);
  ones_disp       : out std_logic_vector(6 downto 0)
  );
end top;

architecture behaviour of top is

component double_dabble is
  port (
    result_padded           : in  std_logic_vector(11 downto 0); 
    ones                    : out std_logic_vector(3 downto 0);
    tens                    : out std_logic_vector(3 downto 0);
    hundreds                : out std_logic_vector(3 downto 0)
  );  
end component;

component double_buffer is
  generic (
    bits : integer := 3
  );
  port(
    d     : in  std_logic_vector(bits - 1 downto 0);
    clk   : in  std_logic;
    reset : in  std_logic;
    q     : out std_logic_vector(bits - 1 downto 0)
  );
end component;

component seven_seg is
  port (
    clk             : in  std_logic; 
    reset           : in  std_logic;
    bcd             : in std_logic_vector(3 downto 0);
    seven_seg_out   : out std_logic_vector(6 downto 0)
  );
end component;

component rising_edge_detect is
  port(
    clk   : in  std_logic;
    reset : in  std_logic;
    d     : in  std_logic;
    edge  : out std_logic
  );
end component;

component alu is
  port (
    clk           : in  std_logic;
    reset         : in  std_logic;
    a             : in  std_logic_vector(7 downto 0); 
    b             : in  std_logic_vector(7 downto 0);
    op            : in  std_logic_vector(1 downto 0); -- 00: add, 01: sub, 10: mult, 11: div
    result        : out std_logic_vector(7 downto 0)
  );  
end component;

component ram is
  generic (addr_width : integer := 2;
           data_width : integer := 4);
  port (
    clk               : in std_logic;
    reset             : in std_logic;
    we                : in std_logic;
    addr              : in std_logic_vector(addr_width - 1 downto 0); -- := "01";
    din               : in std_logic_vector(data_width - 1 downto 0); -- := (1 => '1', others => '0');
    dout              : out std_logic_vector(data_width - 1 downto 0)
  );
end component;

component fsm is
  port(
    clk       : in  std_logic;
    reset     : in  std_logic;
    exec      : in  std_logic;
    op_in     : in  std_logic_vector(3 downto 0);
    loopback  : in  std_logic_vector(7 downto 0);
    result    : in  std_logic_vector(7 downto 0);
    we        : out std_logic;
    op_out    : out std_logic_vector(1 downto 0);
    ram_addr  : out std_logic_vector(1 downto 0);
    ram_input : out std_logic_vector(7 downto 0);
    rom_addr  : out std_logic_vector(4 downto 0)
  );
end component;

component blink_rom is
  port(
    clock : in  std_logic;
    address: in  std_logic_vector(4 downto 0);
    q: out  std_logic_vector(3 downto 0)
  );
end component;

signal padded_output : std_logic_vector(11 downto 0);
signal buffered_input : std_logic_vector(7 downto 0);
signal reg_input : std_logic_vector(7 downto 0) := (others => '0');
signal raw_result, loop_input : std_logic_vector(7 downto 0) := (others => '0');
signal raw_ones, raw_tens, raw_hundreds : std_logic_vector(6 downto 0);
signal address: std_logic_vector(1 downto 0) := "00";
signal buffered_op  : std_logic_vector(1 downto 0);
signal major_op_code: std_logic_vector(3 downto 0);
signal rom_address : std_logic_vector(4 downto 0);
signal bcd_ones, bcd_tens, bcd_hundreds : std_logic_vector(3 downto 0);
signal reset, write_enable, counter_iter, exec_z, buffered_exec, exec_trigger : std_logic;

begin

reset <= not reset_n;
padded_output <= "0000" & loop_input;

button_buffer : process(reset,clk) is
begin
  if (reset = '1') then
    exec_z <= '0';
    buffered_exec <= '0';
  elsif (rising_edge(clk)) then
    exec_z <= exec_btn;
    buffered_exec <= exec_z;
  end if;
end process;

state_machine: fsm
  port map(
    clk => clk,
    reset => reset,
    exec => exec_trigger,
    op_in => major_op_code,
    loopback => loop_input,
    result => raw_result,
    we => write_enable,
    op_out => buffered_op,
    ram_addr => address,
    ram_input => reg_input,
    rom_addr => rom_address
  );


input_switch_buff : double_buffer
  generic map(
    bits => 8
  )
  port map(
    clk => clk,
    reset => reset,
    d => input_switches,
    q => buffered_input
  );

exec_sense: rising_edge_detect
  port map(
    clk => clk,
    reset => reset,
    d => buffered_exec,
    edge => exec_trigger
  );

registers: ram
  generic map(
    addr_width => 2,
    data_width => 8
  )
  port map(
    clk => clk,
    reset => reset,
    we => write_enable,
    addr => address,
    din => reg_input,
    dout => loop_input
  );

op_order: blink_rom
  port map(
    clock => clk,
    address => rom_address,
    q => major_op_code
  );

main_logic: alu
  port map(
    clk => clk,
    reset => reset,
    a => loop_input,
    b => buffered_input,
    op => buffered_op,
    result => raw_result
  );

bcd: double_dabble
  port map(
    result_padded => padded_output,
    ones => bcd_ones,
    tens => bcd_tens,
    hundreds => bcd_hundreds
  );

ones_convert: seven_seg
  port map(
    clk => clk,
    reset => reset,
    bcd => bcd_ones,
    seven_seg_out => raw_ones
  );

tens_convert: seven_seg
  port map(
    clk => clk,
    reset => reset,
    bcd => bcd_tens,
    seven_seg_out => raw_tens
  );

hundreds_convert: seven_seg
  port map(
    clk => clk,
    reset => reset,
    bcd => bcd_hundreds,
    seven_seg_out => raw_hundreds
  );

ones_buff : double_buffer
  generic map(
    bits => 7
  )
  port map(
    clk => clk,
    reset => reset,
    d => raw_ones,
    q => ones_disp
  );

tens_buff : double_buffer
  generic map(
    bits => 7
  )
  port map(
    clk => clk,
    reset => reset,
    d => raw_tens,
    q => tens_disp
  );

hundreds_buff : double_buffer
  generic map(
    bits => 7
  )
  port map(
    clk => clk,
    reset => reset,
    d => raw_hundreds,
    q => hundreds_disp
  );

end behaviour;
